"""
OpenNEM Compare Data

Compare JSON outputs between OpenNEM versions to check for consistency
"""


import json

from opennem.utils.http import http


def compare_data() -> None:

    return None


if __name__ == "__main__":
    compare_data()
